define([
  "sitecore",
  "jquery",
  "/-/speak/v1/komfoconnector/messageBarUtils.js",
  "/-/speak/v1/komfoconnector/validationUtils.js",
  "/-/speak/v1/komfoconnector/ajaxUtils.js",
  "/-/speak/v1/listmanager/SelectLists.js",
  "/-/speak/v1/campaignmanager/antiForgeryUtils.js",
  "/-/speak/v1/komfoconnector/ConfirmationDialog.js"
], function (
  _sc, $, messageBarUtils, validationUtils, ajaxUtils, selectLists, antiForgeryUtils, confirmationDialog) {

  var customAudienceControllerUrl = "/sitecore/api/komfoconnector/customaudience";
  
  var CustomAudience = _sc.Definitions.App.extend({    
    Events: {
      OnAddList: "komfoconnector:customaudience:add:list",
      OnRemoveList: "komfoconnector:customaudience:remove:list",
      OnAddCampaign: "komfoconnector:customaudience:add:campaign",
      OnRemoveCampaign: "komfoconnector:customaudience:remove:campaign",
      OnCreateCampaign: "komfoconnector:customaudience:create:campaign",
      OnSelectCampaignDialogOk: "komfoconnector:customaudience:selectcampaign:ok",
      OnCreateCampaignDialogClose: "komfoconnector:customaudience:createcampaign:close",
      OnCreateCustomAudience: "komfoconnector:customaudience:create:customaudience",
      OnRemoveCustomAudience: "komfoconnector:customaudience:remove:customaudience",
      OnCancelRemoveCustomAudience: "komfoconnector:customaudience:cancel:remove:customaudience",
      OnSyncCancel: "komfoconnector:customaudience:sync:cancel",
      OnSyncPause: "komfoconnector:customaudience:sync:pause",
      OnSyncPush: "komfoconnector:customaudience:sync:push",
      OnSyncResume: "komfoconnector:customaudience:sync:resume",
      OnOpenEA: "komfoconnector:customaudience:open:analytics"
    },

    Messages: {
      CustomAudienceIsCreated: {
        type: "notification",
        text: 'Custom audience posted to Komfo. Once status changes to "Posted", it will be available for use in your Komfo account.',
        id: "{FE1848A6-1D3F-4E85-8EAF-E6038776C232}"
      },
      CustomAudienceIsRemoved: {
        type: "notification",
        text: 'Request to delete custom audience sent to Komfo. On approval, custom audience will be deleted. Related list and campaign will be unaffected.',
        id: "{790E9DEA-AB25-41DC-AC1B-A623707C15BF}"
      },
      CustomAudienceCannotBeDeleted: {
        type: "notification",
        text: 'Custom audience cannot be deleted as it is currently in use by Komfo for active advertising activity.',
        id: "{8B8D1CB1-5EED-4613-B712-073C6DAEA04F}"
      },
      GeneralError: {
        type: "error",
        text: "Cannot create custom audience. Please contact your administrator.",
        id: "{3B34EB44-DD1C-4FA4-B95C-68205E9BDC90}"
      },
      ActionGeneralError: {
        type: "error",
        text: "Cannot perform action. Please contact your administrator.",
        id: "{C78BE7B6-6436-45A5-B265-FC11D2D2D25C}"
      },
      ActionIsNotAvailableError: {
        type: "error",
        text: "Action unavailable. Refresh page to view current status.",
        id: "{544945DE-21E3-4D6F-8589-F8E515877D8F}"
      },
      CustomAudienceNotFoundError: {
        type: "error",
        text: "The custom audience is deleted or does not exist.",
        id: "{59FC495E-F9C5-4E8D-84EA-0F38657C10D3}"
      },
      CannotRetrieveKomfoClients: {
        type: "error",
        text: "Cannot retrieve Komfo clients. Please contact your administrator.",
        id: "{F9E37087-FE1C-45D3-AF20-773F1968E556}"
      },
      AtLeastOneFieldToBeMatchedShouldBeSelected: {
        type: "error",
        text: "At least one field to be matched should be selected.",
        id: "{0084ABA1-8036-4B84-96A3-CB8EFD513C10}"
      },
      CustomAudienceFailedSyncAndRetryIsPending: {
        type: "warning",
        text: 'The custom audience synchronization has failed and retry is pending. Check log for details.',
        id: "{B87217AF-FEBA-48CD-B255-ACDE0E8B3ADB}"
      },
      CustomAudienceFailedSyncAndPaused: {
        type: "warning",
        text: 'The custom audience synchronization is paused due to repeated failures. Check log for details. Resume sync if issue resolved.',
        id: "{D2905E65-4406-4269-89A4-5823D8372006}"
      },
      CustomAudienceIsPushed: {
        type: "notification",
        text: 'Custom audience is now scheduled to synchronize as soon as possible.',
        id: "{661113BE-5765-4E26-AD47-CD954C7B586D}"
      },
      CustomAudienceIsCanceled: {
        type: "notification",
        text: 'Custom audience is now scheduled to cancel synchronization as soon as possible.',
        id: "{42DD6F18-3EC2-4945-90D7-E7E2C3B0C43A}"
      },
      ContactListIsLocked: {
        type: "warning",
        text: 'This list is locked. No custom audience updates will occur until the list  is unlocked.',
        id: "{E0CCBB25-919F-4F2F-BD92-5941D832CB2A}"
      },
      ContactListIsDeleted: {
        type: "warning",
        text: 'Please note that the list associated with this custom audience has been deleted. You can create campaigns with this custom audience, but it is not possible to make any further updates.',
        id: "{8DB37B59-EF59-4820-BA0A-629E852ADF2C}"
      },
      CampaignIsDeleted: {
        type: "warning",
        text: 'Note that the campaign associated with this custom audience has been deleted. You may still create campaigns in Komfo with this custom audience, but any activity will not be trackedby Sitecore.',
        id: "{50EBCD53-4488-4B3D-895E-B744EE453568}"
      },
      NoCampaignAssignedDeleted: {
        text: 'No campaign assigned (deleted)',
        id: "{040F72C3-0A99-4DDF-9F75-A46C7264568E}"
      },
      NoListAssignedDeleted: {
        text: 'No list assigned (deleted)',
        id: "{CD93ECAB-FF43-4134-A64D-84520C2761E5}"
      },
      MinUpdateFrequencyDays: {
        type: "error",
        text: 'Minimum update frequency is {0} day(s). Please select a higher frequency.',
        id: "{6260EAFB-6B37-4E0D-8855-8F7A78C3609E}"
      },
      MinUpdateFrequencyHours: {
        type: "error",
        text: 'Minimum update frequency is {0} hour(s). Please select a higher frequency.',
        id: "{413E4713-CA8D-47EF-936F-8BC2116E4123}"
      },
      MinUpdateFrequencyMinutes: {
        type: "error",
        text: 'Minimum update frequency is {0} minute(s). Please select a higher frequency.',
        id: "{9C9013D3-B66A-494D-A797-84AEE12703A1}"
      }
    },

    Settings: {
      CustomAudienceSyncMaxNumberOfFailures: "Sitecore.KomfoConnector.CustomAudienceSync.MaxNumberOfFailures",
      CustomAudienceSyncMinUpdateFrequencyMinutes: "Sitecore.KomfoConnector.CustomAudienceSync.MinUpdateFrequencyMinutes"
    },
    
    Actions: {
      Delete: "D79FFC148132446F9891A33321FB0635",
      CancelDelete: "9E03442CAD3D42BFAEC09F9E90CEF631",
      Cancel: "39D741F97BDF466B9E16DBD94476DB01",
      Pause: "E40BAE9249DB4BC0B463240E6DF5727A",
      Push: "39E90B8A25C845238EC750D85B050CC6",
      Resume: "646710938AD14F0F955FDCEAA842A487",
      OpenEA: "1F3623AF5E81485992E6262B860208E2"
    },

    MessageTypes: {
      FromAction: "fromAction",
      FromError: "fromError",
      FromStatus: "fromStatus"
    },

    CampaignTemplateId: "{94FD1606-139E-46EE-86FF-BC5BF3C79804}",

    CustomAudienceModel: {
      Id: 0,
      FieldsToBeMatched: 0
    },

    initialized: function () {
      this.messageList = { fromAction:[], fromStatus:[], fromError:[]};

      this.initializeMessageTranslations();
      this.retrieveMessageTranslations();

      this.initializeCustomAudienceModelBindings();
      this.initializeKomfoClientsDataProvider();
      this.initializeUpdateFrequencyPeriodsDataProvider();
      this.initializeCampaignDataSource();
      this.initializeCreateCampaignDialog();
      this.retrieveKomfoClients();
      this.retrieveUpdateFrequencyPeriods();
      if (window.location.search && window.location.search.indexOf("id=") > 0) {
        this.initializeExistingCustomAudience();
      } else {
        this.SaveButton.set("isVisible", true);
        this.HeaderTitle.set("isVisible", true);
        this.MainBorder.set("isVisible", true);
        this.initializeCustomAudienceListActions();
        this.initializeCampaignActions();
        this.initializeSaveButton();
      }
    },

    getSetting: function (settingKey) {
      return this.SettingsDictionary.get(settingKey);
    },
    
    initializeCustomAudienceActions: function () {
      this.on(this.Events.OnRemoveCustomAudience, this.removeCustomAudience, this);
      this.on(this.Events.OnCancelRemoveCustomAudience, this.cancelRemoveCustomAudience, this);
      this.on(this.Events.OnSyncCancel, function() {
        confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Cancel, confirmationDialog.Buttons.OkCancel, function() {
          this.makeSyncAction("cancel");
        });
      }, this);
      this.on(this.Events.OnSyncPause, function() {
        confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Pause, confirmationDialog.Buttons.OkCancel, function() {
          this.makeSyncAction("pause");
        });
      }, this);
      this.on(this.Events.OnSyncPush, function() {
        confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Push, confirmationDialog.Buttons.OkCancel, function() {
          this.makeSyncAction("push");
        });
      }, this);
      this.on(this.Events.OnSyncResume, function() {
        confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Resume, confirmationDialog.Buttons.OkCancel, function() {
          this.makeSyncAction("resume");
        });
      }, this);
      this.on(this.Events.OnOpenEA, this.openAnalytics, this);
    },

    initializeCustomAudienceListActions: function() {
      selectLists.init(this);
      this.on(this.Events.OnAddList, this.addCustomAudienceList, this);
      this.on(this.Events.OnRemoveList, this.removeCustomAudienceList, this);
    },

    initializeCampaignActions: function() {
      this.on(this.Events.OnAddCampaign, this.addCampaign, this);
      this.on(this.Events.OnRemoveCampaign, this.removeCampaign, this);
      this.on(this.Events.OnCreateCampaign, this.createCampaign, this);
    },

    initializeCustomAudienceModelBindings: function() {

      this.SetupNameValue.on("change:text", function () {
        this.CustomAudienceModel.Name = this.SetupNameValue.get("text");
      }, this);

      this.SetupDescriptionValue.on("change:text", function () {
        this.CustomAudienceModel.Description = this.SetupDescriptionValue.get("text");
      }, this);

      this.CustomAudienceModel.KomfoClientId = this.SetupKomfoClientValue.get("selectedValue");
      this.SetupKomfoClientValue.on("change:selectedValue", function () {
        this.CustomAudienceModel.KomfoClientId = this.SetupKomfoClientValue.get("selectedValue");
      }, this);

      // custom audience list
      this.SetupCustomAudienceListControl.on("change:items", function () {
        var items = this.SetupCustomAudienceListControl.get("items");
        this.CustomAudienceModel.ContactListId = (items == null || items.length === 0)
          ? null
          : items[0].Id;
      }, this);

      // update frequency
      this.CustomAudienceModel.UpdateFrequencyEnabled = this.SetupUpdateFrequencyReRead.get("isChecked");
      this.SetupUpdateFrequencyReRead.on("change:isChecked", function () {
        this.CustomAudienceModel.UpdateFrequencyEnabled = this.SetupUpdateFrequencyReRead.get("isChecked");
        this.SetupUpdateFrequencyReReadValue.set("isEnabled", this.CustomAudienceModel.UpdateFrequencyEnabled && this.CustomAudienceModel.Id === 0);
        this.SetupUpdateFrequencyReReadPeriodValue.set("isEnabled", this.CustomAudienceModel.UpdateFrequencyEnabled && this.CustomAudienceModel.Id === 0);
      }, this);

      this.SetupUpdateFrequencyReReadValue.on("change:text", function () {
        this.CustomAudienceModel.UpdateFrequency = this.SetupUpdateFrequencyReReadValue.get("text");
      }, this);

      this.CustomAudienceModel.UpdateFrequencyPeriod = this.SetupUpdateFrequencyReReadPeriodValue.get("selectedValue");
      this.SetupUpdateFrequencyReReadPeriodValue.on("change:selectedValue", function () {
        this.CustomAudienceModel.UpdateFrequencyPeriod = this.SetupUpdateFrequencyReReadPeriodValue.get("selectedValue");
      }, this);

      // fields to be matched
      if (this.SetupFieldsToBeMatchedEmailCheckBox.get("isChecked") === true) {
        this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedEmailCheckBox.get("value"), 10);
      }
      this.SetupFieldsToBeMatchedEmailCheckBox.on("change:isChecked", function() {
        if (this.SetupFieldsToBeMatchedEmailCheckBox.get("isChecked") === true) {
          this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedEmailCheckBox.get("value"), 10);
        } else {
          this.CustomAudienceModel.FieldsToBeMatched ^= parseInt(this.SetupFieldsToBeMatchedEmailCheckBox.get("value"), 10);
        }
      }, this);

      if (this.SetupFieldsToBeMatchedTelephoneCheckBox.get("isChecked") === true) {
        this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedTelephoneCheckBox.get("value"), 10);
      }
      this.SetupFieldsToBeMatchedTelephoneCheckBox.on("change:isChecked", function () {
        if (this.SetupFieldsToBeMatchedTelephoneCheckBox.get("isChecked") === true) {
          this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedTelephoneCheckBox.get("value"), 10);
        } else {
          this.CustomAudienceModel.FieldsToBeMatched ^= parseInt(this.SetupFieldsToBeMatchedTelephoneCheckBox.get("value"), 10);
        }
      }, this);

      if (this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("isChecked") === true) {
        this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("value"), 10);
      }
      this.SetupFieldsToBeMatchedFacebookIdCheckBox.on("change:isChecked", function () {
        if (this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("isChecked") === true) {
          this.CustomAudienceModel.FieldsToBeMatched |= parseInt(this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("value"), 10);
        } else {
          this.CustomAudienceModel.FieldsToBeMatched ^= parseInt(this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("value"), 10);
        }
      }, this);

      // campaign list
      this.SetupCampaignListControl.on("change:items", function () {
        var items = this.SetupCampaignListControl.get("items");
        this.CustomAudienceModel.CampaignId = (items == null || items.length === 0)
          ? null
          : items[0].Id;
      }, this);
    },

    initializeKomfoClientsDataProvider: function () {
      this.SetupKomfoClientsDataProvider.on("change:data", function () {
        this.SetupKomfoClientsDataProvider.set("isBusy", false);
      }, this);

      this.SetupKomfoClientsDataProvider.on("error", function (error) {
        this.SetupKomfoClientsDataProvider.set("isBusy", false);
        this.addMessage(this.Messages.CannotRetrieveKomfoClients, this.MessageTypes.FromError);
        if (this.MessageBar.get("isVisible") !== true) {
          this.MessageBar.set("isVisible", true);
        }
      }, this);
    },

    initializeUpdateFrequencyPeriodsDataProvider: function () {
      this.SetupUpdateFrequencyReReadPeriodsDataProvider.on("change:data", function () {
        var items = _.map(this.SetupUpdateFrequencyReReadPeriodsDataProvider.get("data"), function (itemData) {
          return new _sc.Definitions.Data.Item(itemData);
        });
        this.SetupUpdateFrequencyReReadPeriodValue.set("items", items);
        this.SetupUpdateFrequencyReReadPeriodsDataProvider.set("isBusy", false);
      }, this);

      this.SetupUpdateFrequencyReReadPeriodsDataProvider.on("error", function (error) {
        this.SetupUpdateFrequencyReReadPeriodsDataProvider.set("isBusy", false);
      }, this);
    },

    initializeCampaignDataSource: function () {
      var that = this;
      this.SetupCampaignDataSource = antiForgeryUtils.getDecoratorFor(this.SetupCampaignDataSource);

      this.SetupCampaignDataSource.on("change:entityID", function() {
        that.SetupCampaignDataSource.set("isBusy", true);
      });

      this.SetupCampaignDataSource.on("change:entity", function () {
        that.SetupCampaignDataSource.set("isBusy", false);
        var entity = that.SetupCampaignDataSource.get("entity");
        entity == null
          ? that.SetupCampaignListControl.set("items", [])
          : that.SetupCampaignListControl.set("items", [entity]);
      });
    },

    initializeCreateCampaignDialog: function() {
      var that = this;
      this.SetupCreateCampaignFrame.viewModel.$el.load(function () {
        // tricky way
        var newCampaignPage = $(that.SetupCreateCampaignFrame.viewModel.$el[0].contentDocument);
        newCampaignPage.find(".sc-globalHeader").hide();
        newCampaignPage.find(".sc-applicationHeader").css("padding", "0px");
        newCampaignPage.find(".sc-applicationHeader-row1").hide();
        newCampaignPage.find("button[data-sc-id=BackButton]").hide();
        newCampaignPage.find("div[data-sc-id=CampaignExecutionAdvancedExpander]").hide();
        newCampaignPage.find("div[data-sc-id=ActionControl] ul").hide();
      });
    },

    initializeSaveButton: function () {
      this.on(this.Events.OnCreateCustomAudience, this.saveCustomAudience, this);
    },
    
    saveCustomAudience: function() {
      this.clearErrorMessages();
      if (this.validateSetupTabData()) {
        confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Save, confirmationDialog.Buttons.OkCancel, function () {
          this.clearStatusMessages();
          if (this.validateSetupTabData()) {
            this.createCustomAudience();
          }
        });
      }
    },

    selectCampaign: function(callback) {
      var that = this;

      // to prevent duplicates running (Cancel/X)
      this.off(this.Events.OnSelectCampaignDialogOk);

      this.once(this.Events.OnSelectCampaignDialogOk, function () {
        this.SetupSelectCampaignDialog.hide();
        if (callback) {
          var selectedNode = that.SetupSelectCampaignsTreeView.get("selectedNode");
          if (selectedNode != null && selectedNode.rawItem != null && selectedNode.rawItem.$templateId === this.CampaignTemplateId) {
            callback(selectedNode.rawItem.itemId);
          }
        }
      }, this);

      this.SetupSelectCampaignDialog.show();
    },

    createCACampaign: function (callback) {
      // to prevent duplicates running (Cancel/X)
      this.off(this.Events.OnCreateCampaignDialogClose);

      this.once(this.Events.OnCreateCampaignDialogClose, function () {
        this.SetupCreateCampaignDialog.hide();
        if (callback) {
          var campaignId = _sc.Helpers.url.getQueryParameters(this.SetupCreateCampaignFrame.viewModel.$el[0].contentDocument.URL).id;
          if (typeof campaignId != "undefined" && campaignId != null)
            callback(campaignId);
          }
      }, this);

      this.SetupCreateCampaignDialog.show();
    },

    addCustomAudienceList: function() {
      var that = this;
      selectLists.SelectList(function (itemId, item) {
        if (typeof item != "undefined" && item != null) {
          that.SetupCustomAudienceListControl.set("items", [item]);
          that.invertActionControlActions(that.SetupCustomAudienceListActionControl);
        }
      });
    },

    removeCustomAudienceList: function () {
      this.SetupCustomAudienceListControl.set("items", []);
      this.invertActionControlActions(this.SetupCustomAudienceListActionControl);
    },

    addCampaign: function () {
      this.selectCampaign($.proxy(this.fillCampaignListControl, this));
    },

    removeCampaign: function () {
      this.SetupCampaignListControl.set("items", []);
      this.invertActionControlActions(this.SetupCampaignActionControl);
    },

    createCampaign: function () {
      var that = this;
      this.createCACampaign(function (selectedItemId) {
        if (typeof selectedItemId != "undefined" && selectedItemId != null) {
          that.invertActionControlActions(that.SetupCampaignActionControl);
          that.SetupCampaignDataSource.set("entityID", selectedItemId);
        }
      });
    },

    invertActionControlActions: function(actionControl) {
      Array.prototype.forEach.call(actionControl.viewModel.actions(), function (action) {
        action.isEnabled() ? action.disable() : action.enable();
      });
    },

    validateSetupTabData: function () {
      var that = this;
      this.hasValidationError = false;

      if (validationUtils.validateRequiredField(this, this.addValidationMessage, validationUtils.Messages.FieldIsRequired, this.CustomAudienceModel.Name, this.SetupNameLabel)) {
        if (validationUtils.validateMaxLength(this, this.addValidationMessage, validationUtils.Messages.MaxValueLength, this.CustomAudienceModel.Name, this.SetupNameLabel, this.SetupNameValue.get("maxLength"))) {
          validationUtils.validateCharacters(this, this.addValidationMessage, validationUtils.Messages.InvalidCharacters, this.CustomAudienceModel.Name, this.SetupNameLabel, this.getSetting(validationUtils.Settings.ItemNameValidation));
        }
      };

      if (validationUtils.validateRequiredField(this, this.addValidationMessage, validationUtils.Messages.FieldIsRequired, this.CustomAudienceModel.Description, this.SetupDescriptionLabel)) {
        validationUtils.validateMaxLength(this, this.addValidationMessage, validationUtils.Messages.MaxValueLength, this.CustomAudienceModel.Description, this.SetupDescriptionLabel, this.SetupDescriptionValue.get("maxLength"));
      };

      validationUtils.validateRequiredField(this, this.addValidationMessage, validationUtils.Messages.FieldValueShouldBeSelected, this.CustomAudienceModel.KomfoClientId, this.SetupKomfoClientLabel);

      if (this.SetupUpdateFrequencyReRead.get("isChecked") === true) {
        var updateFrequencyValid = false;
        var updateFrequencyPeriodValid = false;

        if (validationUtils.validateRequiredField(this, this.addValidationMessage, validationUtils.Messages.FieldIsRequired, this.CustomAudienceModel.UpdateFrequency, this.SetupUpdateFrequencyLabel)) {
          updateFrequencyValid = validationUtils.validatePositiveIntegerField(this, this.addValidationMessage, validationUtils.Messages.FieldIsPositiveInteger, this.CustomAudienceModel.UpdateFrequency, this.SetupUpdateFrequencyLabel);
        }

        updateFrequencyPeriodValid = validationUtils.validateRequiredField(this, this.addValidationMessage, validationUtils.Messages.FieldIsRequired, this.CustomAudienceModel.UpdateFrequencyPeriod, this.SetupUpdateFrequencyReReadPeriodLabel);

        if (updateFrequencyValid && updateFrequencyPeriodValid) {
          var updateFrequencyMinutes = parseInt(this.CustomAudienceModel.UpdateFrequency, 10) * parseInt(this.CustomAudienceModel.UpdateFrequencyPeriod);
          var minUpdateFrequencyMinutes = parseInt(this.getSetting(this.Settings.CustomAudienceSyncMinUpdateFrequencyMinutes), 10);
          if (updateFrequencyMinutes < minUpdateFrequencyMinutes) {
            if (minUpdateFrequencyMinutes % (24 * 60) === 0) {
              this.addValidationMessage(this.Messages.MinUpdateFrequencyDays, function(translatedText) {
                return _sc.Helpers.string.format(translatedText, minUpdateFrequencyMinutes / (24 * 60));
              });
            } else if (minUpdateFrequencyMinutes % 60 === 0) {
              this.addValidationMessage(this.Messages.MinUpdateFrequencyHours, function(translatedText) {
                return _sc.Helpers.string.format(translatedText, minUpdateFrequencyMinutes / 60);
              });
            } else {
              this.addValidationMessage(this.Messages.MinUpdateFrequencyMinutes, function (translatedText) {
                return _sc.Helpers.string.format(translatedText, minUpdateFrequencyMinutes);
              });
            }
          }
        }
      }

      if (this.CustomAudienceModel.FieldsToBeMatched === 0) {
        this.addValidationMessage(this.Messages.AtLeastOneFieldToBeMatchedShouldBeSelected);
      }

      if (typeof this.CustomAudienceModel.ContactListId == "undefined" || this.CustomAudienceModel.ContactListId == null) {
        this.addValidationMessage(validationUtils.Messages.FieldValueShouldBeSelected, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, that.SetupCustomAudienceListExpander.get("header").replace(':', ''));
        });
      }

      if (typeof this.CustomAudienceModel.CampaignId == "undefined" || this.CustomAudienceModel.CampaignId == null) {
        this.addValidationMessage(validationUtils.Messages.FieldValueShouldBeSelected, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, that.SetupCampaignExpander.get("header").replace(':', ''));
        });
      }
      
      if (this.hasValidationError) {
        this.MessageBar.set("isVisible", true);
        return false;
      }

      return true;
    },

    retrieveKomfoClients: function () {
      this.SetupKomfoClientsDataProvider.set("isBusy", true);
      this.SetupKomfoClientsDataProvider.viewModel.getData({
        parameters: {}
      });
    },

    retrieveUpdateFrequencyPeriods: function () {
      this.SetupUpdateFrequencyReReadPeriodsDataProvider.set("isBusy", true);
      this.SetupUpdateFrequencyReReadPeriodsDataProvider.viewModel.getData({
        parameters: {}
      });
    },

    retrieveMessageTranslations: function () {
      this.CommonMessagesDataProvider.viewModel.getData({
        parameters: {}
      });

      this.ValidationMessagesDataProvider.viewModel.getData({
        parameters: {}
      });
    },

    createCustomAudience: function() {
      var that = this;
      
      this.CreateCustomAudienceProgressIndicator.set("isBusy", true);
      this.SaveButton.set("isEnabled", false);

      $.ajax(ajaxUtils.createPostOptions(customAudienceControllerUrl, this.CustomAudienceModel))
        .done(function (response) {
          that.afterCreateCustomAudienceSucceed(response);
        })
        .fail(function () {
          that.addMessage(that.Messages.GeneralError, that.MessageTypes.FromError);
          that.SaveButton.set("isEnabled", true);
        })
        .always(function () {
          that.CreateCustomAudienceProgressIndicator.set("isBusy", false);
        });
    },
    
    afterCreateCustomAudienceSucceed: function (response) {
      this.clearActionMessages();
      this.addMessage(this.Messages.CustomAudienceIsCreated, this.MessageTypes.FromAction);
      this.setExistingState();
      this.initializeCustomAudienceActions();
      var id = response.data.customAudienceId;
      if (id) {
        this.CustomAudienceModel.Id = id;
        history.pushState("", "", "CustomAudience?id=" + id);
        var url = customAudienceControllerUrl + "/" + id + "/Status";
        this.SetupCustomAudienceStatusGenericDataProvider.set("dataUrl", url);        
        this.SetupCustomAudienceStatusGenericDataProvider.viewModel.getData({
          parameters: {},
          onSuccess: $.proxy(this.customAudienceStatusDataProviderSuccessHandler, this)
        });
      }
    },

    removeCustomAudience: function () {
      var that = this;

      confirmationDialog.showConfirmation(this, confirmationDialog.Types.Warning, confirmationDialog.Messages.Delete, confirmationDialog.Buttons.OkCancel, function () {

        this.CreateCustomAudienceProgressIndicator.set("isBusy", true);        

        this.addStatusHandlers($.ajax(ajaxUtils.deleteOptions(customAudienceControllerUrl + "/" + this.CustomAudienceModel.Id)), function () {
          that.addMessage(that.Messages.CustomAudienceIsRemoved, that.MessageTypes.FromAction);
        });

      });
    },

    cancelRemoveCustomAudience: function () {
      this.CreateCustomAudienceProgressIndicator.set("isBusy", true);

      this.addStatusHandlers($.ajax(ajaxUtils.putOptions(customAudienceControllerUrl + "/" + this.CustomAudienceModel.Id + "/cancel/delete")));
    },

    makeSyncAction: function (action) {
      var that = this, message;
      if (action == "cancel") {
        message = that.Messages.CustomAudienceIsCanceled;
      }
      if (action == "push") {
        message = that.Messages.CustomAudienceIsPushed;
      }
      if (action) {
        this.CreateCustomAudienceProgressIndicator.set("isBusy", true);
        this.addStatusHandlers($.ajax(ajaxUtils.putOptions(customAudienceControllerUrl + "/" + this.CustomAudienceModel.Id + "/" + action + "/sync")), function () {
          if (message) that.addMessage(message, that.MessageTypes.FromAction);
        });
      }
    },

    addStatusHandlers: function (request, doneCallback) {
      var that = this;
      this.clearErrorMessages();
      request.done(function (response) {
        that.clearActionMessages();
        if (doneCallback)
          doneCallback.call();
        var url = customAudienceControllerUrl + "/" + that.CustomAudienceModel.Id + "/Status";
        that.SetupCustomAudienceStatusGenericDataProvider.set("dataUrl", url);
        that.SetupCustomAudienceStatusGenericDataProvider.once("error", function (error) {
          if (error && error.response && error.response.status) that.customAudienceErrorHandler(error.response.status);
        }, that);
        that.SetupCustomAudienceStatusGenericDataProvider.viewModel.getData({
          parameters: {},
          onSuccess: $.proxy(that.customAudienceStatusDataProviderSuccessHandler, that)
        });
      })
      .fail(function (data) {
        that.ActionErrorHandler(data);
      })
      .always(function () {
        that.CreateCustomAudienceProgressIndicator.set("isBusy", false);
      });
    },

    setExistingState: function () {
      this.HeaderTitle.set("text", this.SetupNameValue.get("text"));
      this.SetupCustomAudienceListActionControl.set("isVisible", false);
      this.SetupCampaignActionControl.set("isVisible", false);
      this.CustomAudienceActionControl.set("isVisible", true);
      this.CustomAudienceActionControl.set("isEnabled", true);
      this.SaveButton.set("isVisible", false);
      this.SetupNameValue.set("isEnabled", false);
      this.SetupDescriptionValue.set("isEnabled", false);
      this.SetupKomfoClientValue.set("isEnabled", false);
      this.SetupUpdateFrequencyOneTime.set("isEnabled", false);
      this.SetupUpdateFrequencyReRead.set("isEnabled", false);
      this.SetupUpdateFrequencyReReadValue.set("isEnabled", false);
      this.SetupUpdateFrequencyReReadPeriodValue.set("isEnabled", false);
      this.SetupFieldsToBeMatchedEmailCheckBox.set("isEnabled", false);
      this.SetupFieldsToBeMatchedTelephoneCheckBox.set("isEnabled", false);
      this.SetupFieldsToBeMatchedFacebookIdCheckBox.set("isEnabled", false);
      this.InfoBorder.set("isVisible", true);
      this.HeaderTitle.set("isVisible", true);
    },
    
    setCustomAudienceActionActionsState: function (status) {
      _.each(this.CustomAudienceActionControl.viewModel.actions(), function (action) {
        var shouldEnable, shouldDisable;
        var actionState;
        if (action.id() === this.Actions.Delete && status.CustomAudience && status.CustomAudience.Delete) {
          actionState = status.CustomAudience.Delete;
        }
        if (action.id() === this.Actions.CancelDelete && status.CustomAudience && status.CustomAudience.CancelDelete) {
          actionState = status.CustomAudience.CancelDelete;
        }
        if (action.id() === this.Actions.Cancel && status.Synchronization && status.Synchronization.Cancel) {
          actionState = status.Synchronization.Cancel;
        }
        if (action.id() === this.Actions.Pause && status.Synchronization && status.Synchronization.Pause) {
          actionState = status.Synchronization.Pause;
        }
        if (action.id() === this.Actions.Push && status.Synchronization && status.Synchronization.Push) {
          actionState = status.Synchronization.Push;
        }
        if (action.id() === this.Actions.Resume && status.Synchronization && status.Synchronization.Resume) {
          actionState = status.Synchronization.Resume;
        }
        if (action.id() === this.Actions.OpenEA && status.CustomAudience && status.CustomAudience.ExperienceAnalytics
          && status.CustomAudience.ExperienceAnalytics.ExperienceAnalyticsCampaignReportUrl) {
          actionState = status.CustomAudience.ExperienceAnalytics;
          this.ExperienceAnalyticsCampaignReportUrl = status.CustomAudience.ExperienceAnalytics.ExperienceAnalyticsCampaignReportUrl;
        }
        shouldEnable = actionState && actionState.IsEnabled && actionState.IsEnabled != action.isEnabled();
        shouldDisable = actionState && !actionState.IsEnabled && actionState.IsEnabled != action.isEnabled();
        if (shouldEnable)
          action.enable();
        if (shouldDisable)
          action.disable();
      }, this);
    },
    
    setCustomAudienceStatusMessages: function (status) {

      if (status.CustomAudienceStatus && status.CustomAudienceStatus.ContactListLocked === true) {
        this.addMessage(this.Messages.ContactListIsLocked, this.MessageTypes.FromStatus);
      };

      if (status.CustomAudienceStatus && status.CustomAudienceStatus.ContactListExists !== true) {
        this.addMessage(this.Messages.ContactListIsDeleted, this.MessageTypes.FromStatus);
      };

      if (status.CampaignStatus && status.CampaignStatus.CampaignExists !== true) {
        this.addMessage(this.Messages.CampaignIsDeleted, this.MessageTypes.FromStatus);
      };

      if (status.CustomAudienceStatus && status.CustomAudienceStatus.PostingStatus
        && status.CustomAudienceStatus.PostingStatus.PostingState === 2 && status.CustomAudienceStatus.PostingStatus.PostingProgress === 3
        && status.CustomAudienceStatus.PostingStatus.KomfoStatusCode === 234)
        this.addMessage(this.Messages.CustomAudienceCannotBeDeleted, this.MessageTypes.FromStatus);

      if (status.CustomAudienceStatus && status.CustomAudienceStatus.SyncStatus && status.CustomAudienceStatus.SyncStatus.SyncProgress === 3) {
        var numberOfFailures = status.CustomAudienceStatus.SyncStatus.NumberOfFailures;
        var maxFailures = this.getSetting(this.Settings.CustomAudienceSyncMaxNumberOfFailures);
        if (numberOfFailures >= maxFailures)
          this.addMessage(this.Messages.CustomAudienceFailedSyncAndPaused, this.MessageTypes.FromStatus);
        else
          this.addMessage(this.Messages.CustomAudienceFailedSyncAndRetryIsPending, this.MessageTypes.FromStatus);
      }
    },
    
    fillCampaignListControl: function (selectedItemId) {
      if (typeof selectedItemId != "undefined" && selectedItemId != null) {
        this.invertActionControlActions(this.SetupCampaignActionControl);
        if (selectedItemId !== this.SetupCampaignDataSource.get("entityID")) {
          this.SetupCampaignDataSource.set("entityID", selectedItemId);
        } else {
          this.SetupCampaignDataSource.trigger("change:entity");
        }
      }
    },
    
    fillCustomAudienceStatus: function(data) {
      var dateConverter = _sc.Converters.get("date");
      var dateFormat = "dd/mm/yyyy hh:mm";
      var approximateNextUpdate = data.CustomAudienceStatus.SyncStatus.ApproximateNextUpdate;
      approximateNextUpdate = approximateNextUpdate ? approximateNextUpdate : "-";
      approximateNextUpdate = _sc.Helpers.date.isISO(approximateNextUpdate) ? dateConverter.toStringWithFormat(approximateNextUpdate, dateFormat) : approximateNextUpdate;
      var previousUpdate = data.CustomAudienceStatus.SyncStatus.PreviousUpdate;
      previousUpdate = previousUpdate ? dateConverter.toStringWithFormat(previousUpdate, dateFormat) : "-";
      this.PostingStatusValue.set("text", data.CustomAudienceStatus.PostingStatus.PostingStatusText);
      this.SynchronizationStatusValue.set("text", data.CustomAudienceStatus.SyncStatus.SyncStatusText);
      this.PreviousUpdateValue.set("text", previousUpdate);
      this.ApproximateNextUpdateValue.set("text", approximateNextUpdate);
      this.ApproximateSizeValue.set("text", !data.CustomAudienceStatus.SyncStatus.ApproximateSize ? "-" : data.CustomAudienceStatus.SyncStatus.ApproximateSize);
      this.NumberOfContactsValue.set("text", data.CustomAudienceStatus.NumberOfContacts);
      this.StartDateValue.set("text", data.CampaignStatus.StartDate ? dateConverter.toStringWithFormat(data.CampaignStatus.StartDate, dateFormat) : "");
      this.EndDateValue.set("text", data.CampaignStatus.EndDate ? dateConverter.toStringWithFormat(data.CampaignStatus.EndDate, dateFormat) : "");
      this.DateCreatedValue.set("text", data.CampaignStatus.CreatedDate ? dateConverter.toStringWithFormat(data.CampaignStatus.CreatedDate, dateFormat) : "");
      this.CreatedByValue.set("text", data.CampaignStatus.CreatedBy);
    },

    customAudienceDataProviderSuccessHandler: function (responce) {
      this.clearStatusMessages();
      this.SetupNameValue.set("text", responce.data.CustomAudience.Name);
      this.HeaderTitle.set("text", responce.data.CustomAudience.Name);
      this.SetupDescriptionValue.set("text", responce.data.CustomAudience.Description);
      this.SetupKomfoClientValue.set("selectedValue", responce.data.CustomAudience.KomfoClientId);

      // campaign
      var campaign = responce.data.Campaign;
      campaign["itemId"] = campaign.CampaignId;
      this.SetupCampaignListControl.set("items", [campaign]);

      if (responce.data.Status.CampaignStatus.CampaignExists !== true) {
        // dances with empty message changing: need to pass an empty campaign first (above).
        var noCampaignAssignedDeletedMessage;
        if (messageBarUtils.translation[this.Messages.NoCampaignAssignedDeleted.id]) {
          noCampaignAssignedDeletedMessage = messageBarUtils.translation[this.Messages.NoCampaignAssignedDeleted.id];
        } else {
          noCampaignAssignedDeletedMessage = this.Messages.NoCampaignAssignedDeleted.text;
        }
        this.SetupCampaignListControl.set("empty", noCampaignAssignedDeletedMessage);
        this.SetupCampaignListControl.set("items", []);
      }

      // contact list
      var contactList = responce.data.ContactList;
      contactList["itemId"] = contactList.Id;
      this.SetupCustomAudienceListControl.set("items", [contactList]);

      if (responce.data.Status.CustomAudienceStatus.ContactListExists !== true) {
        // dances with empty message changing: need to pass an empty custom audience first (above).
        var noListAssignedDeletedMessage;
        if (messageBarUtils.translation[this.Messages.NoListAssignedDeleted.id]) {
          noListAssignedDeletedMessage = messageBarUtils.translation[this.Messages.NoListAssignedDeleted.id];
        } else {
          noListAssignedDeletedMessage = this.Messages.NoListAssignedDeleted.text;
        }
        this.SetupCustomAudienceListControl.set("empty", noListAssignedDeletedMessage);
        this.SetupCustomAudienceListControl.set("items", []);
      }

      if (responce.data.CustomAudience.UpdateFrequencyEnabled) {
        this.SetupUpdateFrequencyReRead.set("isChecked", true);
        this.SetupUpdateFrequencyReReadValue.set("text", responce.data.CustomAudience.UpdateFrequency);
        this.SetupUpdateFrequencyReReadPeriodValue.set("selectedValue", responce.data.CustomAudience.UpdateFrequencyPeriod + "");
      } else {
        this.SetupUpdateFrequencyOneTime.set("isChecked", true);
      }

      // fields to be matched
      this.SetupFieldsToBeMatchedEmailCheckBox.set("isChecked", responce.data.CustomAudience.FieldsToBeMatched & parseInt(this.SetupFieldsToBeMatchedEmailCheckBox.get("value", 10)));
      this.SetupFieldsToBeMatchedTelephoneCheckBox.set("isChecked", responce.data.CustomAudience.FieldsToBeMatched & parseInt(this.SetupFieldsToBeMatchedTelephoneCheckBox.get("value", 10)));
      this.SetupFieldsToBeMatchedFacebookIdCheckBox.set("isChecked", responce.data.CustomAudience.FieldsToBeMatched & parseInt(this.SetupFieldsToBeMatchedFacebookIdCheckBox.get("value", 10)));

      this.fillCustomAudienceStatus(responce.data.Status);

      this.InfoBorder.set("isVisible", true);
      this.MainBorder.set("isVisible", true);
      
      this.setCustomAudienceStatusMessages(responce.data.Status);
      this.setCustomAudienceActionActionsState(responce.data.Status.CustomAudienceActions);
      this.showMessages();
    },
    
    customAudienceStatusDataProviderSuccessHandler: function (responce) {
      this.clearStatusMessages();
      this.fillCustomAudienceStatus(responce.data);
      this.setCustomAudienceStatusMessages(responce.data);
      this.setCustomAudienceActionActionsState(responce.data.CustomAudienceActions);
      this.showMessages();

      if (responce.data.CampaignStatus.CampaignExists !== true) {
        this.SetupCampaignListControl.set("empty", messageBarUtils.translation[this.Messages.NoCampaignAssignedDeleted.id]);
        this.SetupCampaignListControl.set("items", []);
      }

      if (responce.data.CustomAudienceStatus.ContactListExists !== true) {
        this.SetupCustomAudienceListControl.set("empty", messageBarUtils.translation[this.Messages.NoListAssignedDeleted.id]);
        this.SetupCustomAudienceListControl.set("items", []);
      }
    },
    
    customAudienceErrorHandler: function (status) {
      if (status === 404) {
        this.CustomAudienceActionControl.set("isVisible", false);
        this.CustomAudienceActionControl.set("isEnabled", false);
        this.SaveButton.set("isVisible", false);
        this.SaveButton.set("isEnabled", false);
        this.InfoBorder.set("isVisible", false);
        this.MainBorder.set("isVisible", false);
        this.HeaderTitle.set("text", this.StringDictionary.translate("Custom audience not found"));
        this.clearActionMessages();
        this.clearStatusMessages();
        this.clearErrorMessages();
        this.addMessage(this.Messages.CustomAudienceNotFoundError, this.MessageTypes.FromError);
      }
    },
    
    ActionErrorHandler: function (data) {
      if (data.status === 403) {
        this.addMessage(this.Messages.ActionIsNotAvailableError, this.MessageTypes.FromError);
      } else if (data.status === 404) {
        this.customAudienceErrorHandler(data.status);
      } else {
        this.addMessage(this.Messages.ActionGeneralError, this.MessageTypes.FromError);
      }
    },
    
    openAnalytics: function () {
      var url = this.ExperienceAnalyticsCampaignReportUrl;
      if (url)
        window.open(url, "_blank");
    },

    initializeExistingCustomAudience: function () {
      var query = window.location.search;
      query = query.replace("?","");
      var id;
      var queryParams = _.chain(query.split('&'))
        .map(function(params) {
          var p = params.split('=');
          return [p[0], decodeURIComponent(p[1])];
        })
        .object()
        .value();
      id = queryParams["id"];
      if (id) {
        this.CustomAudienceModel.Id = id;
        var url = customAudienceControllerUrl + "/" + id;
        this.SetupCustomAudienceGenericDataProvider.set("dataUrl", url);
        this.SetupCustomAudienceGenericDataProvider.once("error", function (error) {
          if (error && error.response && error.response.status) this.customAudienceErrorHandler(error.response.status);
        }, this);
        this.clearErrorMessages();
        this.SetupCustomAudienceGenericDataProvider.viewModel.getData({
          parameters: {},
          onSuccess: $.proxy(this.customAudienceDataProviderSuccessHandler, this)
        });
        this.setExistingState();
        this.initializeCustomAudienceActions();
      } else {
        this.MainBorder.set("isVisible", true);
      }
    },
    
    initializeMessageTranslations: function () {
      this.CommonMessagesDataProvider.once("change:hasData", function () {
        var items = _.map(this.CommonMessagesDataProvider.get("data"), function (itemData) {
          return new _sc.Definitions.Data.Item(itemData);
        });
        messageBarUtils.addTranslation(items);
      }, this);

      this.ValidationMessagesDataProvider.once("change:hasData", function () {
        var items = _.map(this.ValidationMessagesDataProvider.get("data"), function (itemData) {
          return new _sc.Definitions.Data.Item(itemData);
        });
        messageBarUtils.addTranslation(items);
      }, this);
    },
    
    clearErrorMessages: function () {
      this.messageList.fromError = [];
    },
    
    clearStatusMessages: function () {
      this.messageList.fromStatus = [];
    },

    clearActionMessages: function () {
      this.messageList.fromAction = [];
    },
    
    addMessage: function (message, type, formatTextFunc) {
      if (type) {
        var list = this.messageList[type];
        message = _.clone(message);
        if (formatTextFunc) {
          message.formatTextFunc = formatTextFunc;
        }
        list.push(message);
        // show messages after finishing current thread
        this.shouldShowMessage = true;
        setTimeout($.proxy(this.showMessages, this), 0);
      }
    },
    
    addValidationMessage: function (message, formatTextFunc) {
      this.addMessage(message, this.MessageTypes.FromError, formatTextFunc);
      this.hasValidationError = true;
    },

    showMessages: function () {
      var messages = [];
      if (this.shouldShowMessage) {
        messages = messages.concat(this.messageList.fromAction, this.messageList.fromError, this.messageList.fromStatus);

        this.shouldShowMessage = false;
        messageBarUtils.displayMessages(this.MessageBar, messages);

        if (messages.length > 1)
          this.MessageBar.set("expanded", true);
      }
    }
    
  });

  return CustomAudience;
});