define([
  "sitecore",
  "jquery"
], function (
  _sc, $) {

  var openCustomAudienceUrl = "/sitecore/client/Applications/KomfoConnector/CustomAudience?id=";
  
  var FacebookCustomAudiences = _sc.Definitions.App.extend({
    Events: {
      OnSearch: "komfoconnector:customaudiences:search",
      OnNextPage: "komfoconnector:customaudiences:page:next",
      OnOpenCA: "komfoconnector:customaudiences:open:customaudience",
      OnOpenEA: "komfoconnector:customaudiences:open:analytics"
    },

    Actions: {
      OpenCA: "D055B88D772B4A7BAFAED9D8670C60CE",
      OpenEA: "19BAB1F0923948ED9005E69A03A9911F"
    },
    
    search: "",
    listData: [],
    
    initialized: function () {
      this.on(this.Events.OnSearch, this.searchCustomAudiences, this);
      this.on(this.Events.OnNextPage, this.nextPageCustomAudiences, this);
      this.ListOfCustomAudiencesDataProvider.on("error", this.dataProviderErrorHandler, this);
      this.searchCustomAudiences();
      this.ListOfCustomAudiences.on("change:selectedItemId", this.selectCA, this);
      this.on(this.Events.OnOpenCA, this.openCustomAudience, this);
      this.on(this.Events.OnOpenEA, this.openAnalytics, this);
    },

    selectCA: function () {
      var selectedItemId = this.ListOfCustomAudiences.get("selectedItemId");
      var selectedItem = this.ListOfCustomAudiences.get("selectedItem");
      if (selectedItemId && selectedItem) {
        _.each(this.CustomAudiencesActionControl.viewModel.actions(), function(action) {
          if (action.id() === this.Actions.OpenCA && !action.isEnabled()) {
            action.enable();
          }
          if (action.id() === this.Actions.OpenEA && !action.isEnabled() && selectedItem.get("ExperienceAnalyticsCampaignReportUrl")) {
            action.enable();
          }
        }, this);
      }
      else
        _.each(this.CustomAudiencesActionControl.viewModel.actions(), function(action) {
          action.disable();
        }, this);
    },

    openAnalytics: function () {
      var selectedItem = this.ListOfCustomAudiences.get("selectedItem");
      var url = selectedItem.get("ExperienceAnalyticsCampaignReportUrl");
      if (url)
        window.open(url, "_blank");
    },

    openCustomAudience: function () {
      var selectedItemId = this.ListOfCustomAudiences.get("selectedItemId");
      var url = openCustomAudienceUrl + selectedItemId;
      window.location.href = url;
    },

    searchCustomAudiences: function () {
      this.MessageBar.set("isVisible", false);
      this.search = this.SearchButtonTextBox.get("text");
      this.listData = [];
      this.ListOfCustomAudiencesDataProvider.set("pageNumber", 0);
      this.ListOfCustomAudiencesDataProvider.viewModel.getData({ parameters: { search: this.search }, onSuccess: $.proxy(this.dataProviderSuccessHandler, this) });
    },

    nextPageCustomAudiences: function () {
      this.MessageBar.set("isVisible", false);
      var pgn = this.ListOfCustomAudiencesDataProvider.get("pageNumber")+1;
      this.ListOfCustomAudiencesDataProvider.set("pageNumber", pgn);
      this.ListOfCustomAudiencesDataProvider.viewModel.getData({ parameters: { search: this.search }, onSuccess: $.proxy(this.dataProviderSuccessHandler, this) });
    },

    dataProviderErrorHandler: function () {
      this.MessageBar.set("isVisible", true);
      this.ListOfCustomAudiencesDataProvider.set("hasMoreData", false);
      this.ListOfCustomAudiencesDataProvider.set("isBusy", false);
    },

    dataProviderSuccessHandler: function () {
      this.MessageBar.set("isVisible", false);
      this.listData = this.listData.concat(this.transformData(this.ListOfCustomAudiencesDataProvider.get("data")));      
      this.ListOfCustomAudiences.set("items", this.listData);
    },
    
    transformData: function (data) {
      _.each(data, function (el, idx, list) {
        el.ApproximateSize = !el.ApproximateSize ? "-" : el.ApproximateSize;
        el.NumberOfContacts = !el.NumberOfContacts ? "-" : el.NumberOfContacts;
      }, this);

      return data;
    }
  });
  
  return FacebookCustomAudiences;
});