﻿define([
  "sitecore",
  "jquery",
  "/-/speak/v1/komfoconnector/messageBarUtils.js",
  "/-/speak/v1/komfoconnector/validationUtils.js",
  "/-/speak/v1/komfoconnector/wizardTabUtils.js",
  "/-/speak/v1/komfoconnector/ajaxUtils.js"
], function (
  _sc, $, messageBarUtils, validationUtils, wizardTabUtils, ajaxUtils) {
  
  var CreateClientWizard = _sc.Definitions.App.extend({
  
    WizardTabs:  {
      DetailsTabId               : "{9BD65220-2BCF-4ABC-B700-EE109FDC432B}",
      ApplicationCredentialsTabId: "{F257D09F-A6F7-4C53-80EB-27080265B17C}",
      AccountCredentialsTabId    : "{97F79073-3752-4372-BCDB-BB7B63CE09B7}",
      ResultsTabId               : "{5ACD290A-9E69-4FD8-80D2-BBDECB643FD6}"
    },

    HttpStatusCodes:  {
      BadRequest: 400
    },

    KomfoStatusCodes:  {
      InvalidClientCredentials: 50
    },

    Messages: {
      ClientAlreadyExists: {
        type: "error",
        text: "\"{0}\" already exists.",
        id: "{FF233A06-52DD-4402-AD13-FBE0941BF8C5}"
      },
      KomfoClientIsCreated: {
        type: "notification",
        text: "Komfo client is created.",
        id: "{12AFF20A-B68B-4C2D-A5E4-246793A67592}"
      },
      InvalidClientCredentials: {
        type: "error",
        text: "Application ID or Application secret is invalid.",
        id: "{0B8EFCDE-DA22-48BE-9340-39F2BEA6B376}"
      },
      GeneralError: {
        type: "error",
        text: "Unable to create Komfo client. Please, contact your administrator.",
        id: "{4BBCB6F7-E542-401C-8D08-332652188085}"
      }
    },

    Settings: {
      DefaultNumberOfCallsPerMinute: "Sitecore.KomfoConnector.KomfoApi.DefaultNumberOfCallsPerMinute"
    },

    ClientModel: {
      ApplicationCredentials: {},
      AccountCredentials: {}
    },
    
    initialized: function () {

      this.initializeMessageTranslations();
      this.initializeWizardTabControl();
      this.initializeWizardButtons();
      this.initializeDetailsDataProvider();
      this.initializeClientModelBindings();

      this.ApplicationCredentialsCallsPerMinuteValue.set("text", this.getSetting(this.Settings.DefaultNumberOfCallsPerMinute));
    },

    getSetting: function(settingKey) {
      return this.SettingsDictionary.get(settingKey);
    },

    initializeWizardTabControl: function () {
      wizardTabUtils.disableInactiveWizardTabs(this.WizardTabControl);
      this.WizardTabControl.on("change:selectedTab", this.onChangeWizardTab, this);
    },

    onChangeWizardTab: function() {
      var selectedTab = this.WizardTabControl.get("selectedTab");
      wizardTabUtils.enableWizardTab(this.WizardTabControl, selectedTab);

      switch (selectedTab) {
        case this.WizardTabs.DetailsTabId:
          this.PreviousButton.set("isEnabled", false);
          this.NextButton.set("isEnabled", true);
          break;
        case this.WizardTabs.ApplicationCredentialsTabId:
          this.PreviousButton.set("isEnabled", true);
          this.NextButton.set("isEnabled", true);
          break;
        case this.WizardTabs.AccountCredentialsTabId:
          this.ApplicationCredentialsMessageBar.set("isVisible", false);
          this.PreviousButton.set("isEnabled", false);
          this.NextButton.set("isEnabled", false);
          this.retrieveAccountCredentials();
          break;
        case this.WizardTabs.ResultsTabId:
          this.PreviousButton.set("isEnabled", false);
          this.NextButton.set("isVisible", false); // this will show Finish button
          if (!this.ResultsMessageBar.get("hasMessages")) {
            this.FinishButton.set("isEnabled", false);
            this.createClient();
          }
          break;
        default:
          console.error("Undefined tab " + selectedTab);
      }
    },

    initializeWizardButtons: function() {

      this.NextButton.on("click", function () {
        this.validateCurrentTabData();
      }, this);

      this.PreviousButton.on("click", function () {
        wizardTabUtils.switchToWizardTab(this.WizardTabControl, -1);
      }, this);

      this.FinishButton.on("click", function () {
        this.closeDialog();
      }, this);

    },

    validateCurrentTabData: function () {
      switch (this.WizardTabControl.get("selectedTab")) {
        case this.WizardTabs.DetailsTabId:
          if (this.validateDetailsTabData()) {
            this.retrieveClientExists();
          };
          break;
        case this.WizardTabs.ApplicationCredentialsTabId:
          if (this.validateApplicationCredentialsTabData()) {
            wizardTabUtils.switchToWizardTab(this.WizardTabControl, 1);
          };
          break;
        default:
          wizardTabUtils.switchToWizardTab(this.WizardTabControl, 1);
      }
    },

    validateDetailsTabData: function () {
      this.DetailsMessageBar.removeMessages();

      if (validationUtils.validateRequiredField(this, this.addDetailsValidationMessage, validationUtils.Messages.FieldIsRequired, this.ClientModel.Name, this.DetailsNameLabel)) {
        if (validationUtils.validateMaxLength(this, this.addDetailsValidationMessage, validationUtils.Messages.MaxValueLength, this.ClientModel.Name, this.DetailsNameLabel, this.getSetting(validationUtils.Settings.MaxItemNameLength))) {
          validationUtils.validateCharacters(this, this.addDetailsValidationMessage, validationUtils.Messages.InvalidCharacters, this.ClientModel.Name, this.DetailsNameLabel, this.getSetting(validationUtils.Settings.ItemNameValidation));
        }
      };

      if (this.DetailsMessageBar.get("hasMessages")) {
        this.DetailsMessageBar.set("isVisible", true);
        return false;
      }

      return true;
    },

    validateApplicationCredentialsTabData: function () {

      this.ApplicationCredentialsMessageBar.removeMessages();

      validationUtils.validateRequiredField(this, this.addApplicationCredentialsValidationMessage, validationUtils.Messages.FieldIsRequired, this.ClientModel.ApplicationCredentials.ApplicationId, this.ApplicationCredentialsApplicationIdLabel);
      validationUtils.validateRequiredField(this, this.addApplicationCredentialsValidationMessage, validationUtils.Messages.FieldIsRequired, this.ClientModel.ApplicationCredentials.ApplicationSecret, this.ApplicationCredentialsApplicationSecretLabel);
      if (validationUtils.validateRequiredField(this, this.addApplicationCredentialsValidationMessage, validationUtils.Messages.FieldIsRequired, this.ClientModel.CallsPerMinute, this.ApplicationCredentialsCallsPerMinuteLabel)) {
        validationUtils.validatePositiveIntegerField(this, this.addApplicationCredentialsValidationMessage, validationUtils.Messages.FieldIsPositiveInteger, this.ClientModel.CallsPerMinute, this.ApplicationCredentialsCallsPerMinuteLabel);
      };

      if (this.ApplicationCredentialsMessageBar.get("hasMessages")) {
        this.ApplicationCredentialsMessageBar.set("isVisible", true);
        return false;
      }

      return true;
    },

    initializeDetailsDataProvider: function() {
      var that = this;

      this.DetailsDataProvider.on("change:data", function () {
        that.DetailsDataProvider.set("isBusy", false);
        var clientExists = that.DetailsDataProvider.get("data");
        
        if (clientExists === null) {
          return;
        }

        if (clientExists) {
          messageBarUtils.displayMessages(that.DetailsMessageBar, [that.Messages.ClientAlreadyExists], function(translatedText) {
            return _sc.Helpers.string.format(translatedText, that.ClientModel.Name);
          });
        } else {
          wizardTabUtils.switchToWizardTab(that.WizardTabControl, 1);
        }
      });

      this.DetailsDataProvider.on("error", function(error) {
        that.DetailsDataProvider.set("isBusy", false);
        messageBarUtils.displayMessages(that.DetailsMessageBar, [{ type: "error", text: error.message }]);
      });
    },

    retrieveClientExists: function () {
      // Need to null data. Otherwise "change:data" event won't fire in some cases
      this.DetailsDataProvider.set("data", null);
      this.DetailsDataProvider.set("isBusy", true);
      this.DetailsDataProvider.viewModel.getData({
        parameters: {
          clientName: this.ClientModel.Name
        }
      });
    },

    retrieveAccountCredentials: function () {
      var that = this;
      this.AccountCredentialsProgressIndicator.set("isBusy", true);

      var postOptions = ajaxUtils.createPostOptions(
        "/sitecore/api/KomfoConnector/createclient/accountcredentials",
        this.ClientModel.ApplicationCredentials);

      $.ajax(postOptions)
        .done(function (result) {
          that.ClientModel.AccountCredentials.EncryptedAccessToken = result.data.EncryptedAccessToken;
          that.ClientModel.AccountCredentials.ExpiresIn = result.data.ExpiresIn;
          wizardTabUtils.switchToWizardTab(that.WizardTabControl, 1);
        })
        .fail(function (error) {
          if (error && error.status && error.status === that.HttpStatusCodes.BadRequest
            && error.responseJSON
            && error.responseJSON.KomfoStatusCode
            && error.responseJSON.KomfoStatusCode === that.KomfoStatusCodes.InvalidClientCredentials) {
            messageBarUtils.displayMessages(that.ApplicationCredentialsMessageBar, [that.Messages.InvalidClientCredentials]);
            wizardTabUtils.switchToWizardTab(that.WizardTabControl, -1);
          } else {
            messageBarUtils.displayMessages(that.ResultsMessageBar, [that.Messages.GeneralError]);
            wizardTabUtils.switchToWizardTab(that.WizardTabControl, 1);
          }
        })
        .always(function () {
          that.AccountCredentialsProgressIndicator.set("isBusy", false);
        });
    },

    initializeClientModelBindings: function () {
      var that = this;
      this.DetailsNameValue.on("change:text", function() {
        that.ClientModel.Name = that.DetailsNameValue.get("text");
      });
      this.ApplicationCredentialsApplicationIdValue.on("change:text", function () {
        that.ClientModel.ApplicationCredentials.ApplicationId = that.ApplicationCredentialsApplicationIdValue.get("text");
      });
      this.ApplicationCredentialsApplicationSecretValue.on("change:text", function () {
        that.ClientModel.ApplicationCredentials.ApplicationSecret = that.ApplicationCredentialsApplicationSecretValue.get("text");
      });
      this.ApplicationCredentialsCallsPerMinuteValue.on("change:text", function () {
        that.ClientModel.CallsPerMinute = that.ApplicationCredentialsCallsPerMinuteValue.get("text");
      });
    },

    createClient: function () {
      var that = this;
      this.ResultsProgressIndicator.set("isBusy", true);

      $.ajax(ajaxUtils.createPostOptions("/sitecore/api/komfoconnector/createclient", this.ClientModel))
        .done(function () {
          messageBarUtils.displayMessages(that.ResultsMessageBar, [that.Messages.KomfoClientIsCreated]);
        })
        .fail(function () {
          messageBarUtils.displayMessages(that.ResultsMessageBar, [that.Messages.GeneralError]);
        })
        .always(function() {
          that.ResultsProgressIndicator.set("isBusy", false);
          that.FinishButton.set("isEnabled", true);
      });
    },
    
    initializeMessageTranslations: function () {
      this.CommonMessagesDataSource.once("change:hasItems", function () { messageBarUtils.addTranslation(this.CommonMessagesDataSource.get("items")); }, this);
      this.ValidationMessagesDataSource.once("change:hasItems", function () { messageBarUtils.addTranslation(this.ValidationMessagesDataSource.get("items")); }, this);
    },

    addDetailsValidationMessage: function (message, formatTextFunc) {
      messageBarUtils.addMessage(this.DetailsMessageBar, message, formatTextFunc);      
    },

    addApplicationCredentialsValidationMessage: function (message, formatTextFunc) {
      messageBarUtils.addMessage(this.ApplicationCredentialsMessageBar, message, formatTextFunc);
    }

  });

  return CreateClientWizard;
});