define(["sitecore"], function (_sc) {
  "use strict";

  return {

    Messages: {
      FieldIsRequired: {
        type: "error",
        text: "{0} field is required.",
        id: "{5714EA13-CD03-4591-A7AD-01FB1F1DBB4C}"
      },
      FieldValueShouldBeSelected: {
        type: "error",
        text: "{0} should be selected.",
        id: "{A0A8BB0D-AE3A-40D5-A29E-52C80B9B5FE5}"
      },
      FieldIsPositiveInteger: {
        type: "error",
        text: "{0} field should be a positive integer.",
        id: "{CFB747EF-2D57-4977-8F03-B68D55AB2630}"
      },
      InvalidCharacters: {
        type: "error",
        text: "{0} field contains invalid characters.",
        id: "{1619A6EF-9C6A-44A0-91D9-9A437400F600}"
      },
      MaxValueLength: {
        type: "error",
        text: "{0} max length is {1}.",
        id: "{3D1A8355-8F08-4987-BF17-A2307204A057}"
      }
    },

    Settings: {
      ItemNameValidation: "ItemNameValidation",
      MaxItemNameLength: "MaxItemNameLength"
    },

    validateRequiredField: function (app, addMessageHelper, message, fieldValue, fieldLabel) {
      if (!fieldValue || fieldValue.length === 0) {
        
        addMessageHelper.call(app, message, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, fieldLabel.get("text").replace(':', ''));
        });

        return false;
      }

      return true;
    },

    validatePositiveIntegerField: function (app, addMessageHelper, message, fieldValue, fieldLabel) {
      // http://stackoverflow.com/questions/10834796/validate-that-a-string-is-a-positive-integer
      var number = ~~Number(fieldValue);
      if (!(String(number) === fieldValue && number > 0)) {
        addMessageHelper.call(app, message, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, fieldLabel.get("text").replace(':', ''));
        });

        return false;
      }

      return true;
    },

    validateMaxLength: function (app, addMessageHelper, message, fieldValue, fieldLabel, maxLength) {
      if (fieldValue.length > maxLength) {
        addMessageHelper.call(app, message, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, fieldLabel.get("text").replace(':', ''), maxLength);
        });

        return false;
      }

      return true;
    },

    validateCharacters: function (app, addMessageHelper, message, fieldValue, fieldLabel, validCharactersRegex) {
      var re = new RegExp(validCharactersRegex);
      if (!re.test(fieldValue)) {
        addMessageHelper.call(app, message, function (translatedText) {
          return _sc.Helpers.string.format(translatedText, fieldLabel.get("text").replace(':', ''));
        });

        return false;
      }

      return true;
    }
  };
});